function rst

%Code copyright 2004 by M. B. Hamilton and Georgetown University.
%Code may be modified as long as credit is given to the original author.

% program to calculate Rst estimator of Fst based on microsatellite data.
% Includes bootstrap significance testing where resampling is across POPULATIONS

% version 1.3 10/25/2003
% allelic state can be zero, includes weightings for loci, tallies observed
% heterozygosity

% input:
% program expects an Excel file with ...
% 1st row: number of loci; comment
% 2nd row: number of populations; comment
% 3rd row: title of dataset
% 4th row: locus names
% 5th row: name of 1st population
%numeric data starts in 6th row, 5 rows down
%Population names must contain at least one character so that they are NAN.
%Each population must contain at least one individual with non-missing data at each locus.

%declare variables
rawmissingvalue=-9;       %value of <=rawmissingvalue is considered missing data in original data file, should be <zero
stdmissingvalue=-100;    %value of <=stdmissingvalue is considered missing data in standardized data file
bootreps = 500;    %number of bootstrap replicates, must use at least one
sigalpha = 0.05;    %significance level for confidence interval, provides (1-sigalpha)% CI

doWeights = 0;  %switch for weighted average of variance components, 0=skip 1=do calculation
%weights is a 1 by nloci vector of weights used to weight loci in calculating multilocus Rho

weights = [1,1,1,1];	%weights for example file, all loci have equal weight, or could set switch above to skip weights

%weights = [1,0.5,0.5,0.5,0.5,0.25,0.25,0.25,0.25,1,0.1667,0.1667,0.1667,0.1667,0.1667,0.1667,0.5,0.5,1,1,1,1,1]	%weights for C. alta files

%end declare variables

inFile = 'rstexample.xls';    %name of the input Excel file
%outFile = 'rst_output.txt';   %name of tab delimited text file for output

[data, labels] = xlsread(inFile);    %read in data, get numeric data and text data separately

nloci = data(1,1);   %number of loci, top left cell
npops = data(2,1);   %number of populations, one cell below top left cell

%calculate individual population sizes
start = 6;                  %numerical values always start on 6th row
popsizes = zeros(npops,1);  %allocate space
popindices = zeros(npops,2);    %allocate space
vec = isnan(data(:,1));     %get vector of 0 = non-nan, 1 = nan, p. 160 of Hanselman
maxindex = size(vec,1);

vec(maxindex+1,1)=1;    %cram an extra 1 for nan into last cell since it is lacking
maxindex = maxindex+1;  %increment maxindex by one

counter=0;
pop=1;
for i=6:maxindex
    % count rows until encountering a value of 1 (equal to nan).  
    % NANs are population name fields that separate population data blocks.
    if (vec(i,1) == 1)
        popsizes(pop,1) = counter;
        popindices(pop,2)=i-1;  %last row of data for this pop
        popindices(pop,1)=i-counter;  %first row of data for this pop
        counter=0;
        pop=pop+1;
    else
        counter=counter+1;
    end
end
% popsizes
% popindices

% now need to get JUST numerical data into a matrix
rows = sum(popsizes,1); %sum down column
cols = 2*nloci;  %total number of allele entries, 2 per locus
genotypes = zeros(rows,cols);   %allocate space

counter=1;
for i=1:npops
    for j=popindices(i,1):1:popindices(i,2)
        genotypes(counter,:) = data(j,:);
        counter=counter+1;
    end
end
% genotypes

% get observed heterozygosity (Ho) for raw data
Ho=zeros(nloci,1);   %observed heterozygosity per locus, allocate space
[Ho]=getHo(genotypes, cols, rows, rawmissingvalue);
Ho

% make new genotype matrix
% put data into matrix with one column of alleles per locus
stdpopindices=zeros(npops,2);    %allocate space
allelesperlocus = 2*rows;
stdgenotypes = zeros(allelesperlocus,nloci);   %allocate space
for x=1:npops
    countcol=0;
	for i=1:nloci
        if x==1
            countrow=1;
            
        else
            countrow=2*(sum(popsizes(1:x-1)))+1;
        end
        stdpopindices(x,1) = countrow;
            
        for j=1:2
            countcol=countcol+1;
            for k=(sum(popsizes(1:x-1),1)+1):(sum(popsizes(1:x),1))
                %k
                stdgenotypes(countrow, i) = genotypes(k, countcol);
                countrow=countrow+1;
            end
            stdpopindices(x,2) = countrow-1;
        end
	end
end
% stdgenotypes
% popindices
% stdpopindices

%for raw data missing values are <0
% get mean allelic state per locus for raw data
totmeans=zeros(nloci,1);   %mean allelic state per locus for total population, allocate space
missing=zeros(nloci,1);   %number of missing alleles per locus, allocate space
[totmeans,missing]=getmean(stdgenotypes, nloci, allelesperlocus, rawmissingvalue);
%totmeans

% get var and SD in allelic state per locus for raw data
totvars=zeros(nloci,2);   %allocate space
[totvars]=getvariance(stdgenotypes, totmeans, nloci, allelesperlocus, rawmissingvalue);
%totvars

%standardize allelic values for each locus
for i=1:nloci
    for j=1:allelesperlocus
        if (stdgenotypes(j,i)>=0)
            stdgenotypes(j,i) = (stdgenotypes(j,i)-totmeans(i,1))/totvars(i,2); %z=(yi-ybar)/SD Goodman eqn. 3
        else
            stdgenotypes(j,i) = stdmissingvalue;   %set missing to this value since will not be able to use <=0 anymore
        end
    end
end
%stdgenotypes

%get mean and variance for each locus of standardized data, should be mean=0 and var = 1, but will check
stdmeans=zeros(nloci,1);   %mean allelic state per locus for total population, allocate space
stdmissing=zeros(nloci,1);   %number of missing alleles per locus, allocate space
svar=zeros(nloci,2);   %variance and SD of standardized allelic state per locus for total population, allocate space
[stdmeans,stdmissing]=getmean(stdgenotypes, nloci, allelesperlocus, stdmissingvalue);
[svar]=getvariance(stdgenotypes, stdmeans, nloci, allelesperlocus, stdmissingvalue);

%get within population variance
swvar=zeros(nloci,npops);   %variance of standardized allelic state per locus within each population, allocate space
stdmeanswp=zeros(nloci,npops);   %mean allelic state per locus within populations, allocate space
perpopmissing=zeros(nloci,npops);

[swvar, stdmeanswp, perpopmissing]=withinpopvar(stdgenotypes,stdpopindices,npops,nloci,popsizes,stdmissingvalue);

% get between population variance according to Goodman 1997
stdmeans=stdmeans';  %transpose so dimens are 1 by nloci
stdmeanswp=stdmeanswp'; %transpose so dimens are npops by nloci
perpopmissing=perpopmissing';   %transpose so dimens are npops by nloci
allelesperlocus = (2*repmat(popsizes,[1,nloci]))-perpopmissing; %alleles per locus minus missing data
mlstdmeans = repmat(stdmeans,[npops,1]);

squaredDiff = (stdmeanswp-mlstdmeans).^2;
Sbsquared = allelesperlocus.*squaredDiff;  %nd x (ybar-ydoublebar) within each pop for each locus
MSb = sum(Sbsquared,1)/(npops-1);    %sum across pops for each locus and div npops-1
no = (1/(npops-1))*( (sum(allelesperlocus,1)-( sum(allelesperlocus.^2,1)/(sum(allelesperlocus,1) ) ) ) );
mlnpops=repmat(npops,[1,nloci]);
swvar=swvar';    %transpose so dimens are npops by nloci
eplisonMSw = (no./mlnpops).*sum(swvar./allelesperlocus,1);
sigmasquaredB = (MSb-eplisonMSw)./no

swvarbar=zeros(1,nloci);  %average within pop variances in std allelic state, allocate space
swvarbar=(sum(swvar,1))./npops

%get single locus Rho
SingleLocusRho = sigmasquaredB./(sigmasquaredB+swvarbar)   % eqn. 4 of Goodman 1997

%get multi locus Rho
mlavgsigmasquaredB = sum(sigmasquaredB,2)/nloci;
mlavgswvarbar = sum(swvarbar,2)/nloci;
MultilocusRho = mlavgsigmasquaredB/(mlavgsigmasquaredB+mlavgswvarbar)

%get weighted multi locus Rho
if (doWeights == 1)
    WmlavgsigmasquaredB = sum(weights.*sigmasquaredB,2)/nloci;
	Wmlavgswvarbar = sum(weights.*swvarbar,2)/nloci;
	WeightedMultilocusRho = WmlavgsigmasquaredB/(WmlavgsigmasquaredB+Wmlavgswvarbar)
else
    'skipped weighted multilocus Rho calculation'
end

% bootstrap across populations
pops = (1:npops);    %row vector of all populations in sample
pops=pops';          %col vector of all populations in sample
for y=1:bootreps
    [rstdpop,rpopindices,rpopsizes,rallelesperlocus] = resamplepops(nloci,npops,pops,stdgenotypes,stdpopindices,popsizes,allelesperlocus); %sample with replacement from all populations in sample
    
    rrows = sum(rpopsizes,1);
    rallelesperlocus=2.*rrows;
    
    %get mean and variance for each locus of standardized data, should be mean=0 and var = 1, but will check
	stdmeans=zeros(nloci,1);   %mean allelic state per locus for total population, allocate space
	stdmissing=zeros(nloci,1);   %number of missing alleles per locus, allocate space
	svar=zeros(nloci,2);   %variance and SD of standardized allelic state per locus for total population, allocate space
	[stdmeans,stdmissing]=getmean(rstdpop, nloci, rallelesperlocus, stdmissingvalue);
	[svar]=getvariance(rstdpop, stdmeans, nloci, rallelesperlocus, stdmissingvalue);
	
	%get within population variance
	swvar=zeros(nloci,npops);   %variance of standardized allelic state per locus within each population, allocate space
	stdmeanswp=zeros(nloci,npops);   %mean allelic state per locus within populations, allocate space
	perpopmissing=zeros(nloci,npops);
	[swvar, stdmeanswp, perpopmissing]=withinpopvar(rstdpop,rpopindices,npops,nloci,rpopsizes,stdmissingvalue);

	% get between population variance according to Goodman 1997
	stdmeans=stdmeans';  %transpose so dimens are 1 by nloci
	stdmeanswp=stdmeanswp'; %transpose so dimens are npops by nloci
	perpopmissing=perpopmissing';   %transpose so dimens are npops by nloci
	rallelesperlocus = (2*repmat(rpopsizes,[1,nloci]))-perpopmissing; %alleles per locus minus missing data
	mlstdmeans = repmat(stdmeans,[npops,1]);
	
	squaredDiff = (stdmeanswp-mlstdmeans).^2;
	Sbsquared = rallelesperlocus.*squaredDiff;  %nd x (ybar-ydoublebar) within each pop for each locus
	MSb = sum(Sbsquared,1)/(npops-1);    %sum across pops for each locus and div npops-1
	no = (1/(npops-1))*( (sum(rallelesperlocus,1)-( sum(rallelesperlocus.^2,1)/(sum(rallelesperlocus,1) ) ) ) );
	mlnpops=repmat(npops,[1,nloci]);
	swvar=swvar';    %transpose so dimens are npops by nloci
	eplisonMSw = (no./mlnpops).*sum(swvar./rallelesperlocus,1);
	sigmasquaredB = (MSb-eplisonMSw)./no;
	
	swvarbar=zeros(1,nloci);  %average within pop variances in std allelic state, allocate space
	swvarbar=(sum(swvar,1))./npops;
	    
    %get single locus Rho
	bRho(y,:) = sigmasquaredB./(sigmasquaredB+swvarbar);   % eqn. 4 of Goodman 1997
	
	%get multi locus Rho
	mlavgsigmasquaredB = sum(sigmasquaredB,2)/nloci;
	mlavgswvarbar = sum(swvarbar,2)/nloci;
	mlbRho(y) = mlavgsigmasquaredB/(mlavgsigmasquaredB+mlavgswvarbar);
	
	%get weighted multi locus Rho
    if (doWeights == 1)
        WmlavgsigmasquaredB = sum(weights.*sigmasquaredB,2)/nloci;
		Wmlavgswvarbar = sum(weights.*swvarbar,2)/nloci;
		WmlbRho(y) = WmlavgsigmasquaredB/(WmlavgsigmasquaredB+Wmlavgswvarbar);
    end

end

[SingleLocusRhoLowerCIbounds(:,1), SingleLocusRhoUpperCIbounds(:,1)] = bootconfint(bRho',sigalpha,bootreps)
[MultilocusRhoLowerCIbounds(:,1), MultilocusRhoUpperCIbounds(:,1)] = bootconfint(mlbRho,sigalpha,bootreps)
if (doWeights == 1)
    [WeightedMultilocusRhoLowerCIbounds(:,1), WeightedMultilocusRhoUpperCIbounds(:,1)] = bootconfint(WmlbRho,sigalpha,bootreps)
end

%plot histrogram of resampled Fst estimates used for CI
figure(1);
bins = -0.1:0.05:1.1;
hist(mlbRho(1,:), bins);
title('distribution of bootstrap (populations resampled) multilocus Rho estimates'); 
xlabel('bootstrap mlRho value');
ylabel('frequency');


%_______________________________________
function [rstdpop,rpopindices,rpopsizes,rallelesperlocus] = resamplepops(nloci,npops,pops,stdgenotypes,stdpopindices,popsizes,allelesperlocus)
%resample populations with replacement for bootstrap
%test to see if resampled pops are all the same population, if so resample again
%cannot assume anything about uniformity of pop size

%inputs:
%npops = the number of populations sampled
%pops = the row vector of indices of sampled populations
%stdgenotypes = a npop by nloci matrix of standardized allele values (all
%               alleles are in one column for each locus)
%stdpopindices = a npop by 2 matrix of the rows for the beginning (col 1) and end (col 2) of each population

    %get a sample with replacement
    dup=1;
    while dup==1
		index = unidrnd(npops,1,npops); % get random set of npops indices
        mx=max(index);   %get first index of maximum value
        i=find(index==mx);   %get all other indices that are equal to maximum value
        numdups = length(i);   %get size of array of indices that are equal to max
        if numdups==npops
            dup=1;  %all pops are the same
        else
            dup=0;  %all pops are not the same
        end
    end
    
    index=index';   %make index a column vector
    temppopindices=stdpopindices(index,:);
    rpopsizes = popsizes(index,:);
    rallelesperlocus = allelesperlocus(index,:);
    
    rrows=sum(rpopsizes,1); %resampled total individuals (sum down column)
    rstpop=zeros(rrows,nloci);  %allocate space
    startrow=1;
    for i=1:npops
        %startrow
        span=2*rpopsizes(i);
        rstdpop(startrow:startrow+span-1,:) = stdgenotypes(temppopindices(i,1):temppopindices(i,2),:);
        
        rpopindices(i,1)=startrow;  %start for resampled pop i
        rpopindices(i,2)=startrow+span-1;  %end for resampled pop i
        
        startrow=startrow+span;
    end
    %rpopindices

%________________________
function [swvar, stdmeanswp, perpopmissing]=withinpopvar(stdgenotypes,stdpopindices,npops,nloci,popsizes,stdmissingvalue);
%get within population variance
tempswvar=zeros(npops,2);   %variance and SD of standardized allelic state per locus within each population, allocate space
%swvar=zeros(nloci,npops);   %variance of standardized allelic state per locus within each population, allocate space
%stdmeanswp=zeros(nloci,npops);   %mean allelic state per locus within populations, allocate space
%perpopmissing=zeros(nloci,npops);
for i=1:npops
    subdata = stdgenotypes(stdpopindices(i,1):stdpopindices(i,2),:);
    [stdmeanswp(:,i),perpopmissing(:,i)]=getmean(subdata, nloci, 2*popsizes(i,1), stdmissingvalue);
    %stdmeanswp
    [tempswvar]=getvariance(subdata, stdmeanswp(:,i), nloci, 2*popsizes(i,1), stdmissingvalue);
    %ttempswvar=tempswvar';
    swvar(:,i)=tempswvar(:,1);
end
%swvar
%________________________
function [means,nmissing]=getmean(datamatrix, cols, rows, missingval);
%sum to get mean, BUT must skip missing data
%missing data are values <= missingval
%returns means per locus and missing per locus
%means are 1 by nloci
for i=1:cols
    sum=0;
    miss=0;
    for j=1:rows
        if (datamatrix(j,i)>missingval)
            sum=sum+datamatrix(j,i);
        else
            miss=miss+1;
        end
    end
    means(i,1) = sum/(rows-miss);
    nmissing(i,1)=miss;
end
%________________________
function [var]=getvariance(datamatrix, means, cols, rows, missingval);
%get variance, BUT must skip missing data (so can't use var function easily!)
%missing data are values <= missingval
%var is cols (nloci) by 2
%returns variance (col 1) and SD (col 2) of allelic state per locus
for i=1:cols
    ssd=0;
    missing=0;
    for j=1:rows
        if (datamatrix(j,i)>missingval)
            ssd=ssd+(datamatrix(j,i)-means(i,1))^2;
        else
            missing=missing+1;
        end
    end
    var(i,1) = ssd/(rows-missing-1);    %var = ssd/n-1
    var(i,2) = sqrt(var(i,1));          %SD = sqrt(var)
end
%_______________________________________
function [Ho]=getHo(datamatrix, cols, rows, missingval);
%get observed heterozygosity, BUT must skip missing data
%missing data are values <= missingval
%Ho is a column vector nloci by 1
locus=0;
for i=1:2:cols
    locus=locus+1;  %count locus we are on
    numgenotypes=0; %counter for num non-missing genotypes per locus
    CountHo=0;
    for j=1:rows
        if (datamatrix(j,i)>missingval)
            if (datamatrix(j,i)~=datamatrix(j,i+1)) %compare the two alleles for one genotype
                CountHo=CountHo+1;
            end
            numgenotypes=numgenotypes+1;    %increment counter for non-missing genotypes
        end
       
    end
    %numgenotypes
    %CountHo
    Ho(locus,1) = CountHo/numgenotypes; %observed heterozygosity is proprotion of non-missing genotypes which are hets
    %i
    %locus
end
%_______________________________________
function [bootlowbds, bootupbds]=bootconfint(thetareps, alpha, numreps)
% makes bootstrap confidence interval
% inputs:
% fname = name of estimator function (character string; put it in quotes)
% thetareps = vector or array of point estimates from bootstrap procedure
% dimensions: (locus, replicate estimate)
% (may be output of bootdip.m or boothap.m)
% alpha: request (1-alpha) confidence interval
% outputs:
% [bootlowbds, bootupbds] = lower and upper bounds on confidence interval
% dimensions: (locus) 

%numreps=length(thetareps);
% length of confindence interval
intlength = min(round((1-alpha)*numreps),numreps);

% find best confidence interval
% note: for now, we find the SHORTEST INTERVAL in absolute terms.
numloci=size(thetareps);
numloci=numloci(1);

for locus=1:numloci % find CI for each locus separately
thetareps(locus,:)=sort(thetareps(locus,:));
bootlowbd=1;
bootupbd=intlength;
confwidth=thetareps(locus,bootupbd)-thetareps(locus,bootlowbd);
for i=2:numreps-intlength+1
    testwidth=thetareps(locus,i+intlength-1)-thetareps(locus,i);
    if testwidth<confwidth
        confwidth=testwidth;
        bootupbd=i+intlength-1;
        bootlowbd=i;
    end
end

% convert indices to confidence limits
bootlowbds(locus)=thetareps(locus,bootlowbd);
bootupbds(locus)=thetareps(locus,bootupbd);
end

% make column vectors
bootlowbds=bootlowbds.';
bootupbds=bootupbds.';  
%_______________________________________